//---------------------------------------------------------------------
//	File:		main.c
//
//	Written By:	Lawrence Glaister VE7IT
//
// Purpose: This program is used to control an N2PK vector network analyzer.
//          It is designed to run on a dspic30f4011 40pin pic chip
//      
// 
// The following files should be included in the MPLAB project:
//
//		main.c		    -- Main source code file
//      timer1.c        -- timer 1 setup for 1ms intervals
//      serial.c        -- interface to pc serial port
//      encoder.c       -- interface to quadature encoder
//		pwm.c			-- pwn ch for lcd contrast control
//		VNA-lpg.c		-- actual calcs for VNA interface
//		lcd.c			-- interface routines for LCD display
//		p30f4011.gld	-- Linker script file
//---------------------------------------------------------------------
//
// Revision History
//
// March 11 -- first version 
// note: dont make stack too small... it seems to need close to 500 bytes
// 
//---------------------------------------------------------------------- 
#include <stdio.h>
#include "vna.h"

extern void lcd_goto(int);
extern void int_main( void );

//--------------------------Device Configuration------------------------       
// note when using bootloader, the device config does not get written.
// matter of fact, the loader will warn you about it.
// SO... you must use this config as the config for the bootloader image.
// it is left here in case you want to run this image standalone without the 
// bootloader in high memory. see www.ingenia-cat.com

_FOSC(CSW_FSCM_OFF & FRC_PLL8 );  // 7.3mhz *8 / 4 = 14.5mips on internal rc osc
_FWDT(WDT_OFF);                   // wdt off
/* enablebrownout @4.2v, 64ms powerup delay, MCLR pin active */
/* pwm pins in use, both active low to give 64ms charge time on bootstrap caps */
//_FBORPOR(PBOR_ON & BORV_42 & PWRT_64 & MCLR_EN & RST_PWMPIN & PWMxH_ACT_LO & PWMxL_ACT_LO );
_FBORPOR(PBOR_ON & BORV_42 & PWRT_64 & MCLR_EN & RST_IOPIN);

//----------------------------------------------------------------------
extern void setup_TMR1(void);
extern void setup_encoder(void);
extern void setup_uart(void);
extern void setup_lcd(void);
extern void lcd_puts( char *);
extern void lcd_clear( void );


extern volatile unsigned short int timer_test;

extern void setup_pwm(void);
extern void set_pwm(unsigned int amplitude);

extern void setup_adc10(void);

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;
extern volatile int PwrSw;

void __attribute__((__interrupt__ , auto_psv)) _StackError (void)
{
	while (1)
	{
		STATUS_LED = 1;
		STATUS_LED = 0;			
		STATUS_LED = 0;			
		STATUS_LED = 0;			
	}
}

void __attribute__((__interrupt__ , auto_psv)) _AddressError (void)
{
	while (1)
	{
		STATUS_LED = 1;
		STATUS_LED = 1;			
		STATUS_LED = 0;			
		STATUS_LED = 0;			
	}
}

void __attribute__((__interrupt__ , auto_psv)) _MathError (void)
{
	while (1)
	{
		STATUS_LED = 1;
		STATUS_LED = 0;			
		STATUS_LED = 1;			
		STATUS_LED = 0;			
	}
}

int fieldnum = 0;		// global indicator of which field is adjustable

__complex__ double Mo,Ms,Ml,Mt,x,y,z;
long int lastreffreq=0;
long int lasttstfreq=0;
long int lasttrnfreq=0;
long int lastdelf=0;
long int lastminswrfreq1=0;
long int lastminswrfreq2=0;

double Rs,Xs,Cs,Ls;				
double Rp,Xp,Gp,Bp,Cp,Lp;			
double Q;					
double SWR,RL;					
double magrho,angrho,Rerho,Imrho;
double magZu,angZu,magYu,angYu;		

// unknown reading
__complex__ double Mu;		        // raw voltages
__complex__ double rho;			// computed reflection coeff.
__complex__ double Zu,Yu;			// computed impedance
__complex__ double Zo = 50.0+0.0I;

extern volatile unsigned long int timer_auto_off; 
extern volatile unsigned long int timer_bat_chk;

char enShutdown=0;

/*********************************************************************
  Function:        void set-io(void)

  PreCondition:    None.
 
  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        Sets up io ports for in and out

  Note:            Some periperal configs may change them again
********************************************************************/
void setup_io(void)
{
	// PORT B
        ADPCFG = 0xffff;        // make sure analog doesnt grab encoder pins (all digital)
        ADPCFGbits.PCFG0 = 0;   // AN0 is analog in - battery volts
							// 0=output, 1=input
	_TRISB0 = 1;			// used as AN0 above
	_TRISB1 = 0;			// local osc dds to n2pk board
	_TRISB2 = 0;			// rf dds to n2pk board
	_TRISB3 = 0;			// dds reset to n2pk board
	_TRISB4 = 1;			// used by quad encoder input ch A
	_TRISB5 = 1;			// used by quad encoder input ch B
	_TRISB6 = 0;			// wclk to n2pk board
	_TRISB7 = 0;			// dds fqud to n2pk board
	_TRISB8 = 0;			// adcncs to n2pk board

	// PORT C				// 0=output, 1=input
	_TRISC13 = 0;			// serial port aux  tx data
	_TRISC14 = 1;			// serial port aux  rx data
	_TRISC15 = 1;			// spare (maybe future xtal)

	// PORT D				// 0=output, 1=input
	_TRISD0 = 1;			// mode push switch
	_TRISD1 = 1;			// Previous switch
	_TRISD2 = 1;			// Next Switch
	_TRISD3 = 0;			// RFIV Sw 1=Vsensor, 0=Isensor

	// PORT E				// 0=output, 1=input
	_TRISE0 = 1;			// "+External DC Present"
	_TRISE1 = 0;			// lcd D0
	_TRISE2 = 0;			// lcd D1
	_TRISE3 = 0;			// lcd D2
	_TRISE4 = 0;			// lcd D3
	_TRISE5 = 0;			// lcd RS
	_TRISE8 = 0;			// lcd EN

	// PORT F				// 0=output, 1=input
	_TRISF0 = 0;			// adc clock to n2pk board
	_TRISF1 = 0;			// onboard test (heartbeat)led
	_TRISF2 = 1;			// PGC used by icsp
	_TRISF3 = 1;			// PGD used by icsp
	_TRISF4 = 0;			// Soft Pwr Sw Out
	_TRISF5 = 1;			// Soft Pwr Sw In
	_TRISF6 = 1;			// adc shift data out
}

/*********************************************************************
  Function:        int main(void)

  PreCondition:    None.
 
  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        main function of the application. Peripherals are 
                   initialized.

  Note:            None.
********************************************************************/
int main(void) 
{
    // set up status LED
	setup_io();			// make all i/o pins go the right dir
	STATUS_LED = 0;		// led on
	setup_uart();		// setup the serial interface to the PC

        setup_TMR1();       // set up 1ms timer
	IEC0bits.T1IE = 1;  // Enable interrupts for timer 1
						// needed for delays in following routines
	if ( PwrSw > 0 )
	{	
	   //printf("PwrSw active\r\n");
	   _RF4 = 1;
           timer_auto_off=AUTOOFFCNT;   // auto power-off
           timer_bat_chk=BATCHKCNT;     // battery voltage check timer
	}
	//printf("PwrSw inactive\r\n");
	//printf("RE0=%i\r\n",_RE0);					
	// 1/2 seconds startup delay
	//timer_test = 5000;		
	//while ( timer_test );
	
	printf("Powerup..timer...");	
        setup_encoder();    // 16 bit quadrature encoder module setup
	printf("encoder...");
	//ENC_LED = 0;		// turn on encoder led
	printf("adc...");
	setup_adc10();		// setup AN0 to read battery volts
	printf("lcd...");
	setup_lcd();		// set up the lcd display
	lcd_clear();
	lcd_puts("Powerup:      SVNA");
	lcd_goto(0x40);
	lcd_puts(VERSION);
	printf("done\r\n");
	while(PwrSw > 0);		// wait for PwrSw release

	// 2 second delay  so user can read lcd msg's
	timer_test = 20000;
	while(timer_test );
	enShutdown = 1;    // power-up complete, enable soft power-down

	// some junk for the serial channel
	printf("%s%s\n\r",CPWRT,VERSION);
	int_main();		// main menu
	return 0;
}
